@@ -42,6 +42,8 @@
 // Copyright (C) 2019 Christian Persch <chpe@src.gnome.org>
 // Copyright (C) 2019 Oliver Sander <oliver.sander@tu-dresden.de>
 // Copyright (C) 2020 Kai Pastor <dg0yt@darc.de>
+// Copyright (C) 2021 Stefan Löffler <st.loeffler@gmail.com>
+// Copyright (C) 2021 sunderme <sunderme@gmx.de>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -120,18 +122,14 @@ BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
 }
 }
 
-static const char *get_poppler_datadir(void)
+static void get_poppler_localdir(char *retval, const char *suffix)
 {
-    static char retval[MAX_PATH];
-    static int beenhere = 0;
-
     unsigned char *p;
 
-    if (beenhere)
-        return retval;
-
-    if (!GetModuleFileNameA(hmodule, (CHAR *)retval, sizeof(retval) - 20))
-        return POPPLER_DATADIR;
+    if (!GetModuleFileNameA(hmodule, (CHAR *)retval, sizeof(retval) - 20)) {
+        strcpy(retval, POPPLER_DATADIR);
+        return;
+    }
 
     p = _mbsrchr((unsigned char *)retval, '\\');
     *p = '\0';
@@ -140,9 +138,20 @@ static const char *get_poppler_datadir(void)
         if (stricmp((const char *)(p + 1), "bin") == 0)
             *p = '\0';
     }
-    strcat(retval, "\\share\\poppler");
+    strcat(retval, suffix);
+}
 
-    beenhere = 1;
+static const char *get_poppler_datadir(void)
+{
+    static char retval[MAX_PATH];
+    static bool beenhere = false;
+
+    if (beenhere)
+        return retval;
+
+    get_poppler_localdir(retval, "\\share\\poppler");
+
+    beenhere = true;
 
     return retval;
 }
@@ -150,6 +159,25 @@ static const char *get_poppler_datadir(void)
 #    undef POPPLER_DATADIR
 #    define POPPLER_DATADIR get_poppler_datadir()
 
+static const char *get_poppler_fontsdir(void)
+{
+    static char retval[MAX_PATH];
+    static bool beenhere = false;
+
+    if (beenhere)
+        return retval;
+
+    get_poppler_localdir(retval, "\\share\\fonts");
+
+    beenhere = true;
+
+    return retval;
+}
+#    undef POPPLER_FONTSDIR
+#    define POPPLER_FONTSDIR get_poppler_fontsdir()
+
+#else
+#    define POPPLER_FONTSDIR nullptr
 #endif
 
 //------------------------------------------------------------------------
@@ -836,7 +864,7 @@ GooString *GlobalParams::findFontFile(const GooString *fontName)
 {
     GooString *path = nullptr;
 
-    setupBaseFonts(nullptr);
+    setupBaseFonts(POPPLER_FONTSDIR);
     globalParamsLocker();
     const auto fontFile = fontFiles.find(fontName->toStr());
     if (fontFile != fontFiles.end()) {
