@@ -1,6 +1,6 @@
 /* poppler-form.h: qt interface to poppler
  * Copyright (C) 2007-2008, 2011, Pino Toscano <pino@kde.org>
- * Copyright (C) 2008, 2011, 2012, 2015-2021 Albert Astals Cid <aacid@kde.org>
+ * Copyright (C) 2008, 2011, 2012, 2015-2022 Albert Astals Cid <aacid@kde.org>
  * Copyright (C) 2011 Carlos Garcia Campos <carlosgc@gnome.org>
  * Copyright (C) 2012, Adam Reichold <adamreichold@myopera.com>
  * Copyright (C) 2016, Hanno Meyer-Thurow <h.mth@web.de>
@@ -1049,11 +1049,10 @@ SignatureValidationInfo FormFieldSignature::validate(int opt, const QDateTime &v
             priv->range_bounds.append(bound);
         }
     }
-    GooString *checkedSignature = fws->getCheckedSignature(&priv->docLength);
+    const std::optional<GooString> checkedSignature = fws->getCheckedSignature(&priv->docLength);
     if (priv->range_bounds.size() == 4 && checkedSignature) {
         priv->signature = QByteArray::fromHex(checkedSignature->c_str());
     }
-    delete checkedSignature;
 
     return SignatureValidationInfo(priv);
 }
